package de.ipk.batex.persistence.dataAccess;

import java.sql.Connection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.Transaction;

import de.ipk.batex.persistence.extendedModel.TrmCategoryTreeTerm;
import de.ipk.batex.persistence.model.Term;
import de.ipk.batex.persistence.model.TrmCategory;

public class TrmCategoryDao {
	
	private static final String QUERY_HIER= "SELECT "+
											"   * "+ 
											"FROM "+
											"    TRM_CATEGORIES_MV tct  "+
											"CONNECT BY PRIOR tct_id=tct_tct_id "+ 
											"START WITH tct_id = (  SELECT "+
											"                            tct_id "+ 
											"                        FROM "+
											"                            TRM_CATEGORIES_MV "+ 
											"                        WHERE "+
											"                            tct_name= ? "+
											"    ) "+
											"order by level, tct_name";
		

	public List<TrmCategory> getAll(Connection con){

		Session s =HibernateUtil.getSession(con);
		Transaction tx =s.beginTransaction();

		List<TrmCategory> t = s.createCriteria(TrmCategory.class).list();
		tx.commit();
		return  t;
	}
	
	public TrmCategoryTreeTerm getByNameIncludeChilds(String tctName,Connection con){
		return getByNameIncludeChilds(tctName, true,con);
	}
	
	public TrmCategoryTreeTerm getByNameIncludeChilds(String tctName, boolean loadTerms,Connection con){
		
		Session s =HibernateUtil.getSession(con);
		Transaction tx =s.beginTransaction();

		SQLQuery q = s.createSQLQuery(QUERY_HIER).addEntity(TrmCategory.class);
		q.setParameter(0,tctName);  // erstes Element muss root sein!
		List<TrmCategory> cats = q.list();
		tx.commit();
		
		TrmCategoryTreeTerm rootNode = new TrmCategoryTreeTerm(cats.get(0));
		rootNode.setTrmCategoryChilds(getChilds(rootNode, cats,con));
		
		if (loadTerms)
			loadTerms(rootNode,con);
		
		return rootNode;

	}
	
	/**
	 * load all Terms of Category and its childs
	 * 
	 * @param tct
	 */	
	private void loadTerms(TrmCategoryTreeTerm tct,Connection con){
		if (tct.getTerms() == null){
			
			Set<Term> terms = new HashSet<Term>();
			TermDAO dao = new TermDAO();
			terms.addAll(dao.getByCategoryName(tct.getTctName(),con));
			tct.setTerms(terms);
			
			for (TrmCategoryTreeTerm child: tct.getTrmCategoryChilds())
				loadTerms(child,con);
		}
	}

	
	private Set<TrmCategoryTreeTerm> getChilds(TrmCategoryTreeTerm tct, final List<TrmCategory> cats,Connection con) {
		
		Set<TrmCategoryTreeTerm> childs = new HashSet<TrmCategoryTreeTerm>();
		
		for (TrmCategory cat : cats) {
			
			if (tct.getId().equals(cat.getParentTctId())){
				// Child found
				
				TrmCategoryTreeTerm currentNode = new TrmCategoryTreeTerm(cat);
				currentNode.setTrmCategoryParent(tct);
				currentNode.setTrmCategoryChilds(getChilds(currentNode,cats,con));	
				childs.add(currentNode);
			}
		}
		
		return childs;
	}	
}
